Class: Idml::TextEngine::ShapedGlyph

Inherits:
Struct
  • Object
show all
Defined in:
lib/idml/text_engine/shaper.rb

Overview

Converts a text string + font metrics into a sequence of shaped glyphs, each carrying its advance width at the given size.

Instance Attribute Summary collapse

Instance Attribute Details

#codepointObject

Returns the value of attribute codepoint

Returns:

  • (Object)

    the current value of codepoint



7
8
9
# File 'lib/idml/text_engine/shaper.rb', line 7

def codepoint
  @codepoint
end

#is_spaceObject

Returns the value of attribute is_space

Returns:

  • (Object)

    the current value of is_space



7
8
9
# File 'lib/idml/text_engine/shaper.rb', line 7

def is_space
  @is_space
end

#widthObject

Returns the value of attribute width

Returns:

  • (Object)

    the current value of width



7
8
9
# File 'lib/idml/text_engine/shaper.rb', line 7

def width
  @width
end