Class: Idml::Elements::Condition
- Inherits:
-
Lutaml::Model::Serializable
- Object
- Lutaml::Model::Serializable
- Idml::Elements::Condition
- Defined in:
- lib/idml/elements/condition.rb
Overview
<Condition> — a single condition declared in designmap.xml.
Authors tag text runs with one or more conditions (the CSR's
AppliedConditions attribute is a list of condition Self IDs).
The condition's Visible flag controls whether tagged text
appears in the rendered output.
Schema: Condition_Object in
reference-docs/schemas/package/designmap.rnc.