Class: Idml::TextEngine::ShapedGlyph
- Inherits:
-
Struct
- Object
- Struct
- Idml::TextEngine::ShapedGlyph
- Defined in:
- lib/idml/text_engine/shaper.rb
Overview
Converts a text string + font metrics into a sequence of shaped glyphs, each carrying its advance width at the given size.
Instance Attribute Summary collapse
-
#codepoint ⇒ Object
Returns the value of attribute codepoint.
-
#is_space ⇒ Object
Returns the value of attribute is_space.
-
#width ⇒ Object
Returns the value of attribute width.
Instance Attribute Details
#codepoint ⇒ Object
Returns the value of attribute codepoint
7 8 9 |
# File 'lib/idml/text_engine/shaper.rb', line 7 def codepoint @codepoint end |
#is_space ⇒ Object
Returns the value of attribute is_space
7 8 9 |
# File 'lib/idml/text_engine/shaper.rb', line 7 def is_space @is_space end |
#width ⇒ Object
Returns the value of attribute width
7 8 9 |
# File 'lib/idml/text_engine/shaper.rb', line 7 def width @width end |